package artificialLifeSimulator.GUI;

import artificialLifeSimulator.CSS.CSS;
import artificialLifeSimulator.GUI.controls.ParameterSceneFactory;
import artificialLifeSimulator.GUI.parameterScenes.AbstractCarnivoreParameterScene;
import artificialLifeSimulator.GUI.parameterScenes.AbstractHabitatParameterScene;
import artificialLifeSimulator.GUI.parameterScenes.AbstractHerbivoreParameterScene;
import artificialLifeSimulator.GUI.parameterScenes.AbstractPlantParameterScene;
import artificialLifeSimulator.abstractClasses.AbstractCarnivorePopulation;
import artificialLifeSimulator.abstractClasses.AbstractHabitat;
import artificialLifeSimulator.abstractClasses.AbstractHerbivorePopulation;
import artificialLifeSimulator.abstractClasses.AbstractPlant;
import artificialLifeSimulator.abstractClasses.AbstractPlantPopulation;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.layout.GridPane;

/**
 * @author Toni
 * Class implements methods used to create the parameter scene
 */
public class ParameterScene extends Scene{
	/**
	 *  Herbivore parameter scene
	 */
	private AbstractHerbivoreParameterScene herbivoreScene;
	/**
	 *  Carnivore parameter scene
	 */
	private AbstractCarnivoreParameterScene carnivoreScene;
	/**
	 *  Plant parameter scene
	 */
	private AbstractPlantParameterScene plantScene;
	/**
	 * Habitat parameter scene
	 */
	private AbstractHabitatParameterScene habitatScene;

	/**
	 * Method is used to create the parameter scene
	 * @param root Scroll pane in wwere the parameter scene will be set
	 * @param width width of the ecosystem
	 * @param height height of the ecosystem
	 * @param plant chosen plant
	 * @param herbivore chosen herbivore
	 * @param carnivore chosen carnivore
	 * @param habitat chosen habitat
	 * @param goBack return to first scene button
	 * @param startSim simulation starter button
	 */
	public ParameterScene(ScrollPane root, double width, double height, String plant, String herbivore, String carnivore, String habitat,
			Button goBack, Button startSim){
		super(root,width,height);
		herbivoreScene = ParameterSceneFactory.getHerbivoreParameterScene(herbivore);
		carnivoreScene = ParameterSceneFactory.getCarnivoreParameterScene(carnivore);
		plantScene = ParameterSceneFactory.getPlantParameterScene(plant);
		habitatScene = ParameterSceneFactory.getHabitatScene(habitat);
		
		GridPane pane = new GridPane();
		root.setContent(pane);
		
		int currentRow = 0;
		if(habitatScene!=null){
			Label title = new Label(habitat +" Parameters");
			title.setStyle(CSS.boldText);
			title.setId("Title");
			pane.add(title, 0, currentRow++);
			currentRow = habitatScene.createForm(pane, currentRow);
		}
		if(plantScene!=null){
			Label title = new Label(plant +" Parameters");
			title.setStyle(CSS.boldText);
			title.setId("Title");
			pane.add(title, 0, currentRow++);
			currentRow = plantScene.createForm(pane, currentRow);
		}
		if(herbivoreScene!=null){
			Label title = new Label(herbivore +" Parameters");
			title.setStyle(CSS.boldText);
			title.setId("Title");
			pane.add(title, 0, currentRow++);
			currentRow = herbivoreScene.createForm(pane, currentRow);
		}
		if(carnivoreScene!=null){
			Label title = new Label(carnivore +" Parameters");
			title.setStyle(CSS.boldText);
			title.setId("Title");
			pane.add(title, 0, currentRow++);
			currentRow = carnivoreScene.createForm(pane, currentRow);
		}
		pane.add(goBack, 0, currentRow);
		pane.add(startSim, 1, currentRow);
		
		
	}

	/**
	 * @param ecosystemWidth
	 * @param ecosystemHeight
	 * @return the initial abstract plant population
	 */
	public AbstractPlantPopulation InitialPlantPopulation(int ecosystemWidth,int ecosystemHeight){
		if(plantScene == null) return null;
		return plantScene.createPopulation(ecosystemWidth,ecosystemHeight);
	}
	/**
	 * @param ecosystemWidth width of the ecosystem 
	 * @param ecosystemHeight height of the ecosystem
	 * @return the initial abstract carnivore population
	 */
	public AbstractCarnivorePopulation InitialCarnivorePopulation(int ecosystemWidth,int ecosystemHeight){
		if(carnivoreScene == null) return null;
		return carnivoreScene.createPopulation(ecosystemWidth,ecosystemHeight);
	}
	/**
	 * @param ecosystemWidth width of the ecosystem 
	 * @param ecosystemHeight height of the ecosystem
	 * @param plants abstract plant population
	 * @return the initial abstract herbivore population
	 */
	public AbstractHerbivorePopulation InitialHerbivorePopulation(int ecosystemWidth,int ecosystemHeight, AbstractPlantPopulation<AbstractPlant> plants){
		if(herbivoreScene == null) return null;
		return herbivoreScene.createPopulation(ecosystemWidth,ecosystemHeight,plants);
	}
	/**
	 * @param ecosystemWidth width of the ecosystem 
	 * @param ecosystemHeight height of the ecosystem
	 * @return the initial abstract habitat 
	 */
	public AbstractHabitat InitialHabitat(int ecosystemWidth,int ecosystemHeight){
		if(habitatScene == null) return null;
		return habitatScene.create(ecosystemWidth,ecosystemHeight);
	}
	
	

}
