package artificialLifeSimulator.GUI;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.LinkedBlockingQueue;

import artificialLifeSimulator.abstractClasses.AbstractCarnivore;
import artificialLifeSimulator.abstractClasses.AbstractCarnivorePopulation;
import artificialLifeSimulator.abstractClasses.AbstractHabitat;
import artificialLifeSimulator.abstractClasses.AbstractHerbivore;
import artificialLifeSimulator.abstractClasses.AbstractHerbivorePopulation;
import artificialLifeSimulator.abstractClasses.AbstractPlant;
import artificialLifeSimulator.abstractClasses.AbstractPlantPopulation;
import artificialLifeSimulator.helperClasses.AlgorithmConstants;
import artificialLifeSimulator.lifeForms.populations.Biome;
import javafx.application.Application;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.control.SplitPane;
import javafx.scene.layout.GridPane;
import javafx.stage.Stage;

/**
 * @author Toni
 * Class implements methods used to create the simulation scene
 */
public class SimulationScene extends Scene{
	private Biome currentBiome;
	private SimulationControlPane simControl;
	private SimulationPane simPane;
	//Canvas
	private Canvas topLayer;
	private Canvas midLayer;
	private Canvas bottomLayer;
	private GraphicsContext topLayer_g;
	private GraphicsContext midLayer_g;
	private GraphicsContext bottomLayer_g;
	
	private Thread simulationThread;
	private Thread simulationDrawingThread;

	/**
	 * Constructor method used to construct the simulation scene
	 * @param habitat defines the habitat of the organisms
	 * @param plants plant population
	 * @param herbivore herbivore population
	 * @param carnivore carnivore population
	 * @param ecosystemWidth width of the ecosystem
	 * @param ecosystemHeight height of the ecosystem
	 * @param firstScene reference to the first scene where basic information is being set up
	 * @param stage Stage reference
	 * @param width width of the window
	 * @param height height of the window
	 * @param layout parent layout
	 */
	public SimulationScene(AbstractHabitat habitat, AbstractPlantPopulation plants, AbstractHerbivorePopulation herbivore, AbstractCarnivorePopulation carnivore,
			int ecosystemWidth, int ecosystemHeight,Scene firstScene, Stage stage,double width, double height, Parent layout){
		super(layout, width, height);
		BlockingQueue<Biome> drawingQueue = new LinkedBlockingQueue<>(AlgorithmConstants.QueueLimit);
		simPane = new SimulationPane(ecosystemWidth, ecosystemHeight, width-GuiConstants.windowSizeModifierConstant, height, drawingQueue);
		simControl = new SimulationControlPane(this,firstScene, stage);
		SplitPane pane2 = new SplitPane(simPane);
		pane2.setMaxSize(width-GuiConstants.windowSizeModifierConstant, height);
		((GridPane) layout).add(pane2, 0, 0);
		((GridPane) layout).add(simControl, 1, 0);
		
		
		simulationDrawingThread = new Thread(simPane);
		simulationDrawingThread.setDaemon(true);
		
		currentBiome = new Biome(habitat, carnivore, herbivore, plants, ecosystemWidth, ecosystemHeight);
		currentBiome.setBlockingQueue(drawingQueue);
		simulationThread = new Thread(currentBiome);
		simulationThread.setDaemon(true);
	}
	
	public SimulationScene(Biome biome,Scene firstScene, Stage stage,double width, double height, Parent layout){
		super(layout, width, height);
		BlockingQueue<Biome> drawingQueue = new LinkedBlockingQueue<>(AlgorithmConstants.QueueLimit);
		simPane = new SimulationPane(biome.getEcosystemWidth(), biome.getEcosystemHeight(), width-GuiConstants.windowSizeModifierConstant, height, drawingQueue);
		simControl = new SimulationControlPane(this,firstScene, stage);
		SplitPane pane2 = new SplitPane(simPane);
		pane2.setMaxSize(width-GuiConstants.windowSizeModifierConstant, height);
		((GridPane) layout).add(pane2, 0, 0);
		((GridPane) layout).add(simControl, 1, 0);
		
		currentBiome = new Biome(biome.getDomain(),biome.getCarnivores(),biome.getHerbivores(),biome.getPlants(),biome.getEcosystemWidth(),biome.getEcosystemHeight());
		currentBiome.setReferences();
		try{
			simulationDrawingThread = new Thread(simPane);
			simulationDrawingThread.setDaemon(true);
			currentBiome.setBlockingQueue(drawingQueue);
			simulationThread = new Thread(currentBiome);
			simulationThread.setDaemon(true);
		}catch(Exception e){
			
		}
		
		
	}
	
	/**
	 * Returns the current biome
	 * @return current Biome
	 */
	public Biome getCurrentBiome(){
		currentBiome=simPane.getCurrentBiome();
		return currentBiome;
	}
	
	/**
	 *  Method is used to pause the simulation
	 */
	public void blockThreads(){
		simPane.block();
		currentBiome.block();
	}
	
	/**
	 *  Method is used to unpause the simulation
	 */
	public void unblockThreads(){
		try{
			simulationThread.interrupt();
			simulationDrawingThread.interrupt();
		}catch(Exception e){
			
		}
	}
	
	/**
	 *  Method is used to terminate the simulation
	 */
	public void terminateSimulation(){
		simPane.stop();
		currentBiome.stop();
	}
	
	
	/**
	 * Method is used to start the simulation
	 */
	public void startSimulation(){
		try{
			simulationThread.start();
			simulationDrawingThread.start();
		}catch(Exception e){
			
		}
	}
	

}
