package artificialLifeSimulator.GUI.controls;

import javafx.scene.control.IndexRange;
import javafx.scene.control.TextField;

/**
 * @author Toni
 * Class implements text check for input of a integer number
 */
public class IntegerTextField extends TextField{
	private int maxLimit;
	private boolean maxLimitSet;
	
	/**
	 * Constructor used to set the maximal number allowed
	 * @param limit maximum number allowed
	 */
	public IntegerTextField(int limit){
		maxLimit = limit;
		maxLimitSet=true;
	}
	
	/**Constructor sets the initial value of the text field to given parameter
	 * @param text initial text
	 */
	public IntegerTextField(String text){
		super(text);
	}
	
	/**
	 * Default constructor
	 */
	public IntegerTextField(){
		super();
	}
	
	@Override
    public void replaceText(int start, int end, String text)
    {
		try{
			if (validate(text,start,end))
	        {
	            super.replaceText(start, end, text);
	        }
		}catch(NumberFormatException e){
			if(maxLimitSet){
				super.replaceText(0, this.getLength(), Integer.toString(maxLimit));
			}else{
				super.replaceText(0, this.getLength(), Integer.toString(Integer.MAX_VALUE));
			}
		}
        
    }

    @Override
    public void replaceSelection(String text)
    {
        if (validateSelection(text))
        {
            super.replaceSelection(text);
        }
    }



    /**
     * Method is used to validate the given text
     * @param text text
     * @param start start where the text is being set
     * @param end end of the input text
     * @return true if the new text value is a valid integer that satisfies the limitations given to constructor
     */
    private boolean validate(String text,int start,int end)
    {
    	IndexRange range = this.getSelection();
    	String firstPart = this.getText(0,range.getStart());
    	String secondPart =this.getText(range.getEnd(),this.getLength());
    	
    	if(maxLimitSet && Integer.parseInt(firstPart+text+secondPart)>maxLimit){
			throw new NumberFormatException();
    	}
    	
    	Integer.parseInt(firstPart+text+secondPart);
        return (firstPart+text + secondPart).matches("[0-9]*");
    }

    /**
     * Method is used to validate selection
     * @param text text to be validated
     * @return true if the new text value is a valid integer that satisfies the limitations given to constructor
     */
    private boolean validateSelection(String text)
    {
    	if(maxLimitSet && Integer.parseInt(this.getText()+text)>maxLimit){
			throw new NumberFormatException();
		}
    	IndexRange range = this.getSelection();
    	String start = this.getText(0,range.getStart());
    	String end =this.getText(range.getEnd(),this.getLength());
    	Integer.parseInt(start+end);
        return (start+end).matches("[0-9]*");
    }
}
