package artificialLifeSimulator.GUI.parameterScenes.basic;

import artificialLifeSimulator.CSS.CSS;
import artificialLifeSimulator.GUI.controls.IntegerTextField;
import artificialLifeSimulator.GUI.controls.PercentageTextField;
import artificialLifeSimulator.GUI.parameterScenes.AbstractPlantParameterScene;
import artificialLifeSimulator.abstractClasses.AbstractPlantPopulation;
import artificialLifeSimulator.lifeForms.populations.BasicPlantPopulation;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.layout.GridPane;

/**
 * @author Toni
 * Class is used to create basic plant parameter scene
 */
public class BasicPlantParameterScene extends AbstractPlantParameterScene{
	
	private Label maxPlants;
	private IntegerTextField maxPlantsValue;
	
	private Label initialPlantPopulationSize;
	private IntegerTextField initialPlantPopulationSizeValue;
	
	private Label maxEnergy;
	private IntegerTextField maxEnergyValue;
	
	private Label startingEnergy;
	private IntegerTextField startingEnergyValue;
	
	private Label chanceOfReproduction;
	private PercentageTextField chanceOfReproductionValue;
	
	private Label growthRate;
	private IntegerTextField growthRateValue;
	
	/**
	 *  Constructor for basic plant parameter scene
	 */
	public BasicPlantParameterScene(){
		initializeScene();
	}
	@Override
	public AbstractPlantPopulation createPopulation(int ecosystemWidth,int ecosystemHeight) {
		if(!checkFields()){
			return null;
		}
		return new BasicPlantPopulation(Integer.parseInt(maxPlantsValue.getText())
				, Integer.parseInt(initialPlantPopulationSizeValue.getText()), 
				Double.parseDouble(maxEnergyValue.getText()), 
				Double.parseDouble(startingEnergyValue.getText()),
				Double.parseDouble(chanceOfReproductionValue.getText()),
				Double.parseDouble(growthRateValue.getText()),
				ecosystemWidth,
				ecosystemHeight);
	}

	private boolean checkFields() {
		try{
			int popSize1,popSize2;
			double doubleTester1,doubleTester2;
			popSize1=Integer.parseInt(maxPlantsValue.getText());
			popSize2=Integer.parseInt(initialPlantPopulationSizeValue.getText()); 
			if(popSize2>popSize1)
				return false;
			doubleTester1=Double.parseDouble(maxEnergyValue.getText());
			doubleTester2=Double.parseDouble(startingEnergyValue.getText());
			if(doubleTester1<doubleTester2 || popSize1<popSize2){
				return false;
			}
			doubleTester1=Double.parseDouble(chanceOfReproductionValue.getText());
			doubleTester1=Double.parseDouble(growthRateValue.getText());
		}catch(Exception e){
			
			return false;
		}
			return true;
	}
	@Override
	public void initializeScene() {
		
		maxPlants = new Label("Maximum plant population size");
		maxPlantsValue = new IntegerTextField("200");
		
		initialPlantPopulationSize = new Label("Initial plant popoulation size");
		initialPlantPopulationSizeValue = new IntegerTextField("5");
		
		maxEnergy = new Label("Maximum plant energy");
		maxEnergyValue = new IntegerTextField("200");
		

		startingEnergy = new Label("Starting plant energy");
		startingEnergyValue = new IntegerTextField("10");
		
		chanceOfReproduction = new Label("Chance of reproduction");
		chanceOfReproductionValue = new PercentageTextField("0.5");

		growthRate = new Label("Growth rate");
		growthRateValue = new IntegerTextField("2");
		
	}
	@Override
	public int createForm(GridPane pane, int currentRow) {
		pane.setAlignment(Pos.TOP_LEFT);
		pane.setHgap(10);
		pane.setVgap(10);

		pane.add(maxPlants, 0, currentRow);
		pane.add(maxPlantsValue, 1, currentRow++);
		pane.add(initialPlantPopulationSize, 0, currentRow);
		pane.add(initialPlantPopulationSizeValue, 1, currentRow++);
		pane.add(maxEnergy, 0, currentRow);
		pane.add(maxEnergyValue, 1, currentRow++);
		pane.add(startingEnergy, 0, currentRow);
		pane.add(startingEnergyValue, 1, currentRow++);
		pane.add(chanceOfReproduction, 0, currentRow);
		pane.add(chanceOfReproductionValue, 1, currentRow++);
		pane.add(growthRate, 0, currentRow);
		pane.add(growthRateValue, 1, currentRow++);
		
		return currentRow;
		
	}

}
