package artificialLifeSimulator.abstractClasses;

import java.io.Serializable;

import artificialLifeSimulator.helperClasses.Location;
import javafx.scene.canvas.GraphicsContext;

/**
 * @author Toni
 * Class defines methods that are used to simulate the behavior of a carnivore organism
 */
public abstract class AbstractCarnivore implements Serializable{
	/**
	 * Location of the carnivore in the ecosystem
	 */
	protected Location currentLocation;
	
	/**
	 * Method is used to return a new carnivore object with same parameters like the current one
	 * @return new carnivore organism with parameters like this one
	 */
	public abstract AbstractCarnivore newLikeThis();
	/**
	 * Method is used to create a full copy of the carnivore
	 * @return copy of the carnivore
	 */
	public abstract AbstractCarnivore copyOf();
	/**
	 * Method is used to draw the carnivore using the graphics context object
	 * @param g Graphics context used for drawing
	 */
	public abstract void drawMe(GraphicsContext g);
	/**
	 * Method is used to get the location of the carnivore organism
	 * @return location of the carnivore
	 */
	public Location getLocation(){
		return this.currentLocation;
	}
}
