package artificialLifeSimulator.abstractClasses;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.Random;

import javafx.scene.canvas.GraphicsContext;


/**
 * @author Toni
 * Class is defines methods that are used to simulate the behavior of carnivore population in the ecosystem
 *
 * @param <T> Parameter T must be a class that extends the AbstractCarnivore class
 */
public abstract class AbstractCarnivorePopulation<T extends AbstractCarnivore> implements Serializable{
	/**
	 * List of classes that extend AbstractCarnivore class
	 */
	protected LinkedList<T> population;
	
	/**
	 * Method is used to simulate changes of the carnivore population during time in the ecosystem, which are represented as iterations
	 * @param ecosystemWidth ecosystem width
	 * @param ecosystemHeight ecosystem height
	 * @param rand random number generator
	 */
	public abstract void nextIteration(int ecosystemWidth, int ecosystemHeight,Random rand);
	/**
	 * Method is used to create a full copy of the carnivore population
	 * @return copy of the carnivore population
	 */
	public abstract AbstractCarnivorePopulation<T> copyOf();
	/** 
	 * Method is used to get the carnivore population
	 * @return the copy of the population
	 */
	public LinkedList<T> getPopulation(){
		return this.population;
	}
	
	/**
	 * Method is used to draw the abstract carnivore population
	 * @param g Graphics context used to draw the population
	 */
	public void drawPopulation(GraphicsContext g){
		for (AbstractCarnivore temp : population) {
			temp.drawMe(g);
		}
	}
	
}
