package artificialLifeSimulator.abstractClasses;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.Random;

import artificialLifeSimulator.helperClasses.Location;
import artificialLifeSimulator.lifeForms.basic.BasicHerbivore;
import javafx.scene.canvas.GraphicsContext;

/**
 * @author Toni
 * Class defines methods that are used to simulate the behavior of a herbivore organism
 * 
 * @param <T> Parameter T must be a class that extends the AbstractHerbivore class
 */
public abstract class AbstractHerbivore<T extends AbstractHerbivore> implements Serializable{

	/**
	 * Location of the herbivore in the ecosystem
	 */
	protected Location currentLocation;

	/**
	 * Method is used to return a new herbivore object with same parameters like the current one
	 * @return new herbivore organism with parameters like this one
	 */
	public abstract AbstractHerbivore<T> newLikeThis();
	
	/**
	 * Method is used to create a full copy of the herbivore
	 * @return copy of the herbivore
	 */
	public abstract AbstractHerbivore<T> copyOf();
	/**
	 * Method is used to draw the herbivore using the graphics context object
	 * @param g Graphics context used for drawing
	 */
	public abstract void drawMe(GraphicsContext g);
	/**
	 * Method is used to get the location of the herbivore organism
	 * @return location of the herbivore
	 */
	public Location getLocation(){
		return this.currentLocation;
	}
	/**
	 * Method is used to check if the herbivore is dead
	 * @return true if it is dead, false otherwise
	 */
	public abstract boolean isDead();
	
	/**
	 * Method is used to simulate changes of the herbivore during time in the ecosystem, which are represented as iterations
	 * @param ecosystemWidth width of the ecosystem 
	 * @param ecosystemHeight height of the ecosystem
	 * @param rand random number generator
	 * @param closestPlant closest abstract plant
	 * @return List of newly created offsprings
	 */
	public abstract LinkedList<T> nextIteration(int ecosystemWidth, int ecosystemHeight,Random rand, AbstractPlant<AbstractPlant> closestPlant);
}
