package artificialLifeSimulator.abstractClasses;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.Random;

import artificialLifeSimulator.lifeForms.basic.BasicPlant;
import javafx.scene.canvas.GraphicsContext;
/**
 * @author Toni
 * Class is defines methods that are used to simulate the behavior of herbivore population in the ecosystem
 *
 * @param <T> Parameter T must be a class that extends the AbstractHerbivore class
 */
public abstract class AbstractHerbivorePopulation<T extends AbstractHerbivore> implements Serializable{
	/**
	 * List of classes that extend AbstractHerbivore class
	 */
	protected LinkedList<T> population;

	/**
	 *AbstractPlantPopulation in the ecosystem
	 */
	protected transient AbstractPlantPopulation<AbstractPlant> plants;
	/**
	 *AbstractCarnivorePopulation in the ecosystem
	 */
	protected transient AbstractCarnivorePopulation carnivores;
	
	/**
	 * Method is used to simulate changes of the herbivore population during time in the ecosystem, which are represented as iterations
	 * @param ecosystemWidth ecosystem width
	 * @param ecosystemHeight ecosystem height
	 * @param rand random number generator
	 */
	public abstract void nextIteration(int ekosystemWidth, int ekosystemHeight,Random rand);
	/**
	 * Method is used to create a full copy of the herbivore population
	 * @return copy of the herbivore population
	 */
	public abstract AbstractHerbivorePopulation<T> copyOf();
	/** 
	 * Method is used to get the herbivore population
	 * @return the copy of the population
	 */
	public LinkedList<T> getPopulation(){
		return this.population;
	}
	
	
	/** 
	 * Method is used to set the population of plants to the passed argument
	 * @param plants AbstractPlantPopulation that is being set
	 */
	public void setPlants(AbstractPlantPopulation<AbstractPlant> plants){
		this.plants = plants;
	}

	/** 
	 * Method is used to set the population of carnivores to the passed argument
	 * @param plants AbstractCarnivorePopulation that is being set
	 */
	public void setCarnivores(AbstractCarnivorePopulation animalCarnivorePopulation){
		this.carnivores = animalCarnivorePopulation;
	}
	

	/**
	 * Method is used to draw the abstract herbivore population
	 * @param g Graphics context used to draw the population
	 */
	public void drawPopulation(GraphicsContext g){
		for (T temp : population) {
			temp.drawMe(g);
		}
	}
}
