package artificialLifeSimulator.abstractClasses;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.Random;

import artificialLifeSimulator.helperClasses.Location;
import javafx.scene.canvas.GraphicsContext;
/**
 * @author Toni
 * Class defines methods that are used to simulate the behavior of a plant organism
 * 
 * 
 * @param <T> Parameter T must be a class that extends the AbstractPlant class
 */
public abstract class AbstractPlant<T extends AbstractPlant> implements Serializable{
	/**
	 * Location of the plant in the ecosystem
	 */
	protected Location currentLocation;

	/**
	 * Method is used to return a new plant object with same parameters like the current one
	 * @return new plant organism with parameters like this one
	 */
	public abstract AbstractPlant newLikeThis();
	/**
	 * Method is used to create a full copy of the plant
	 * @return copy of the plant
	 */
	public abstract AbstractPlant copyOf();
	/**
	 * Method is used to draw the plant using the graphics context object
	 * @param g Graphics context used for drawing
	 */
	public abstract void drawMe(GraphicsContext g);
	
	/**
	 * Method is used for consuming energy from plants
	 * @param amount defines how much energy wants to be consumed
	 * @return the amount of energy that is successfully consumed
	 */
	public abstract double consume(double amount);
	
	/**
	 * Method is used to simulate the next iteration of plants in the ecosystem
	 * @param ecosystemWidth width of the ecosystem
	 * @param ecosystemHeight height of the ecosystem
	 * @param rand random number generator
	 * @return the list of newly created plants during the iteration
	 */
	public abstract LinkedList<T> nextIteration(int ecosystemWidth, int ecosystemHeight,Random rand);

	/**
	 * Method is used to get the location of the plant organism
	 * @return location of the plant
	 */
	public Location getLocation(){
		return this.currentLocation;
	}

	/**
	 * Method is used to check if the plant is dead
	 * @return true if it is dead, false otherwise
	 */
	public abstract boolean isDead();

}
