package artificialLifeSimulator.lifeForms.basic;

import java.util.LinkedList;
import java.util.Random;

import artificialLifeSimulator.abstractClasses.AbstractPlant;
import artificialLifeSimulator.helperClasses.Location;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;

/**
 * @author Toni
 * Class implements methods that allow the simulation of a basic plant
 *
 */
public class BasicPlant extends AbstractPlant<BasicPlant>{
	private double maxPlantEnergy;
	private double currentEnergy;
	private double chanceOfReproduction;
	private double growthRate;
	private double startingEnergyOffsprings;
	
	/**
	 * Constructor for basic plant object
	 * @param maxPlantEnergy maximum energy
	 * @param startingPlantEnergy starting energy 
	 * @param chanceOfReproduction reproduction chance
	 * @param growthRate energy per iteration
	 * @param loc location of the plant
	 */
	public BasicPlant(double maxPlantEnergy, double startingPlantEnergy, double chanceOfReproduction, double growthRate, Location loc){
		this.maxPlantEnergy=maxPlantEnergy;
		this.currentEnergy=startingPlantEnergy;
		this.chanceOfReproduction=chanceOfReproduction;
		this.startingEnergyOffsprings=startingPlantEnergy;
		this.growthRate=growthRate;
		this.currentLocation = loc;
	}
	
	/**
	 *  Default constructor
	 */
	private BasicPlant(){
		
	}

	@Override
	public AbstractPlant newLikeThis() {
		// TODO Auto-generated method stub
		return null;
	}
	
	@Override
	public  boolean isDead(){
		return this.currentEnergy<=0;
	}

	@Override
	public AbstractPlant copyOf() {
		BasicPlant temp = new BasicPlant();
		temp.maxPlantEnergy=this.maxPlantEnergy;
		temp.currentEnergy=this.currentEnergy;
		temp.chanceOfReproduction=this.chanceOfReproduction;
		temp.growthRate=this.growthRate;
		temp.startingEnergyOffsprings=this.startingEnergyOffsprings;
		temp.currentLocation=this.currentLocation.copyOf();
		return temp;
	}

	@Override
	public void drawMe(GraphicsContext g) {
		double color = currentEnergy/maxPlantEnergy;
		color = (1-color)==0 ? 0.2:color;
		g.setFill(new Color(0, color, 0, 1));
		g.fillOval(this.currentLocation.getX(), this.currentLocation.getY(), 10, 10);
		
	}

	@Override
	public LinkedList<BasicPlant> nextIteration(int ekosystemWidth, int ekosystemHeight,Random rand) {
		if(this.currentEnergy+this.growthRate>this.maxPlantEnergy){
			this.currentEnergy=this.maxPlantEnergy;
		}else{
			this.currentEnergy=this.currentEnergy+this.growthRate;
		}
		if(rand.nextDouble()<this.chanceOfReproduction){
			LinkedList<BasicPlant> newOffsprings = new LinkedList<BasicPlant>();
			double newX,newY;
			newX = Math.max(this.currentLocation.getX()+50-rand.nextInt(100), 0);
			newY = Math.max(this.currentLocation.getY()+50-rand.nextInt(100), 0);
			newX = Math.min(newX, ekosystemWidth);
			newY = Math.min(newY, ekosystemHeight);
			newOffsprings.add(new BasicPlant(this.maxPlantEnergy,startingEnergyOffsprings,this.chanceOfReproduction, this.growthRate,new Location(newX,newY)));
			return newOffsprings;
		}
		return null;
	}

	@Override
	public double consume(double amount) {
		if(this.currentEnergy>amount){
			this.currentEnergy-=amount;
			return amount;
		}else{
			double temp = this.currentEnergy;
			this.currentEnergy=0;
			return temp;
		}
	}

}
