package artificialLifeSimulator.lifeForms.populations;

import java.util.LinkedList;
import java.util.Random;

import artificialLifeSimulator.abstractClasses.AbstractPlant;
import artificialLifeSimulator.abstractClasses.AbstractPlantPopulation;
import artificialLifeSimulator.helperClasses.Location;
import artificialLifeSimulator.lifeForms.basic.BasicPlant;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.control.Label;

/**
 * @author Toni
 * Class implements methods that allow the simulation of a basic plant population
 *
 */
public class BasicPlantPopulation extends AbstractPlantPopulation<BasicPlant> {
	private int maxPopulationSize;
	private int currentPopulationSize;
	

	/**
	 * Constructor for basic plant population
	 * @param maxPlants maximum number of plants in population
	 * @param initialPlantPopulationSize initial population size
	 * @param maxEnergy max energy of the organisms
	 * @param startingEnergy starting energy of organisms
	 * @param chanceOfReproduction reproduction chance of organisms
	 * @param growthRate energy per iteration
	 * @param ecosystemWidth ecosystem width
	 * @param ecosystemHeight ecosystem height
	 */
	public BasicPlantPopulation(int maxPlants, int initialPlantPopulationSize, double maxEnergy,
			double startingEnergy, double chanceOfReproduction, double growthRate, int ecosystemWidth,int ecosystemHeight) {
		Random rand = new Random();
		this.population = new LinkedList<BasicPlant>();
		this.maxPopulationSize = maxPlants;
		this.currentPopulationSize = initialPlantPopulationSize;
		for(int i=0;i<initialPlantPopulationSize;i++){
			population.add(new BasicPlant(maxEnergy, startingEnergy, chanceOfReproduction, growthRate,new Location(rand.nextInt(ecosystemWidth-10), rand.nextInt(ecosystemHeight-10))));
		}
	}
	
	/**
	 *  Default constructor
	 */
	private BasicPlantPopulation(){
		
	}
	

	@Override
	public void nextIteration(int ekosystemWidth, int ekosystemHeight,Random rand) {
		LinkedList<BasicPlant> newPlants = new LinkedList<BasicPlant>();
		LinkedList<BasicPlant> toRemove = new LinkedList<BasicPlant>();
		for(BasicPlant temp : population){
			if(temp.isDead()){
				toRemove.add(temp);
				this.currentPopulationSize--;
				continue;
			}
			LinkedList<BasicPlant> tempOffsprings;
			tempOffsprings=temp.nextIteration(ekosystemWidth, ekosystemHeight, rand);
			if(tempOffsprings != null && this.currentPopulationSize!=this.maxPopulationSize){
				if(this.currentPopulationSize + newPlants.size() <this.maxPopulationSize){
					newPlants.addAll(tempOffsprings);
					this.currentPopulationSize+=tempOffsprings.size();
				}else{
					int populationSlotsLeft = this.maxPopulationSize-this.currentPopulationSize;
					for(int i = 0;i<populationSlotsLeft && i<tempOffsprings.size();i++){
						newPlants.add(tempOffsprings.get(i));
						this.currentPopulationSize++;
					}
				}
			}
		}
		for(BasicPlant temp : toRemove){
			this.population.remove(temp);
		}
		this.population.addAll(newPlants);
		
	}

	@Override
	public AbstractPlantPopulation copyOf() {
		BasicPlantPopulation population = new BasicPlantPopulation();
		population.maxPopulationSize=this.maxPopulationSize;
		population.currentPopulationSize=this.population.size();
		population.population = new LinkedList<BasicPlant>();
		for(BasicPlant temp : this.population){
			population.population.add((BasicPlant)temp.copyOf());
		}
		return population;
	}

}
