package artificialLifeSimulator.GUI;


import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.List;

import artificialLifeSimulator.GUI.controls.IntegerTextField;
import artificialLifeSimulator.GUI.controls.ParameterSceneFactory;
import artificialLifeSimulator.abstractClasses.AbstractCarnivore;
import artificialLifeSimulator.abstractClasses.AbstractCarnivorePopulation;
import artificialLifeSimulator.abstractClasses.AbstractHerbivore;
import artificialLifeSimulator.abstractClasses.AbstractHerbivorePopulation;
import artificialLifeSimulator.abstractClasses.AbstractPlant;
import artificialLifeSimulator.abstractClasses.AbstractPlantPopulation;
import artificialLifeSimulator.helperClasses.AlgorithmConstants;
import artificialLifeSimulator.helperClasses.ImplementedLifeforms;
import artificialLifeSimulator.lifeForms.populations.Biome;
import javafx.application.Application;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.SplitPane;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import javafx.stage.FileChooser;
import javafx.stage.Stage;

/**
 * @author Toni
 * Class is used to define the main window of the application
 */
public class MainFrame extends Application {
	/**
	 * Stage (window) of the application
	 */
	private Stage window;
	/**
	 * References the first scene where the size of ecosystem is being set
	 */
	private Scene firstScene;
	/**
	 *  Parameter scene is where the parameters of each organism is being set up
	 */
	private ParameterScene setParametersScene;
	

	/**
	 *  Wdth of the window
	 */
	private int widthWindow = GuiConstants.defaultWindowWidth;
	
	/**
	 * Height of the window
	 */
	private int heightWindow = GuiConstants.defaultWindowHeight;

	/**
	 * References the simulation scene
	 */
	private SimulationScene simScene;
	/**
	 * References the simulation control pane in the simulation scene
	 */
	private SimulationControlPane simControl;
	/**
	 * References the simulation pane in the simulation scene
	 */
	private SimulationPane simPane;

	/**
	 * Last chosen plant in the first scene
	 */
	private String lastChosenPlant;

	/**
	 * Last chosen herbivore in the first scene
	 */
	private String lastChosenHerbivore;

	/**
	 * Last chosen carnivore in the first scene
	 */
	private String lastChosenCarnivore;

	/**
	 * Last chosen habitat in the first scene
	 */
	private String lastChosenHabitat;


	/**
	 * TextField where ecosystem width is being set
	 */
	private TextField ecosystemDimensionWidth;
	/**
	 * TextField where ecosystem height is being set
	 */
	private TextField ecosystemDimensionHeight;

	@Override
	public void start(Stage primaryStage) throws Exception {
		window = primaryStage;
		GridPane grid = new GridPane();
		grid.setHgap(10);
		grid.setVgap(10);
		grid.setPadding(new Insets(10, 10, 0, 10));

		addFieldsToScene(grid);

		firstScene = new Scene(grid, 450, 250);
		

		firstScene.getStylesheets().add("artificialLifeSimulator/CSS/style.css");
		primaryStage.setTitle("Artificial life simulator");
		primaryStage.setScene(firstScene);
		primaryStage.sizeToScene();
		primaryStage.show();
		window.setResizable(false);
		
	}

	/**
	 * Method is used to create the simulation scene
	 * @return newly created SimulationScene
	 */
	private SimulationScene createSimulationPane() {
		GridPane pane = new GridPane();
		int ecosystemWidth = Integer.parseInt(ecosystemDimensionWidth.getText());
		int ecosystemHeight = Integer.parseInt(ecosystemDimensionHeight.getText());
		if(widthWindow>ecosystemWidth){
			widthWindow=ecosystemWidth;
		}
		if(heightWindow>ecosystemHeight){
			heightWindow = ecosystemHeight;
		}
		AbstractPlantPopulation<AbstractPlant> plants = setParametersScene.InitialPlantPopulation(ecosystemWidth,ecosystemHeight);
		AbstractHerbivorePopulation<AbstractHerbivore> herbivores = setParametersScene.InitialHerbivorePopulation(ecosystemWidth,ecosystemHeight,plants);
		AbstractCarnivorePopulation<AbstractCarnivore> carnivores = setParametersScene.InitialCarnivorePopulation(ecosystemWidth,ecosystemHeight);
		SimulationScene scene = new SimulationScene(setParametersScene.InitialHabitat(ecosystemWidth, ecosystemHeight),
				plants,
				herbivores,
				carnivores,
				ecosystemWidth,
				ecosystemHeight,
				firstScene,
				window,
				widthWindow +GuiConstants.windowSizeModifierConstant, heightWindow, pane);

		try {
			String current = new java.io.File( "." ).getCanonicalPath();
			scene.getStylesheets().add("artificialLifeSimulator/CSS/style.css");
		} catch (IOException e) {

		}
		return scene;
	}

	/**
	 * Method is used to add fields to the first scene
	 * @param grid GridPane where the fields are being added
	 */
	private void addFieldsToScene(GridPane grid) {
		ObservableList<String> optionHabitats = ImplementedLifeforms.getListImplementedHabitats();
		ComboBox<String> habitats = new ComboBox<>(optionHabitats);
		habitats.getSelectionModel().select(0);
		
		ObservableList<String> optionsHerbivores = ImplementedLifeforms.getListImplementedHerbivores();
		ComboBox<String> herbivores = new ComboBox<>(optionsHerbivores);
		herbivores.getSelectionModel().select(0);

		ObservableList<String> optionsCarnivore = ImplementedLifeforms.getListImplementedCarnivores();
		ComboBox<String> carnivores = new ComboBox<>(optionsCarnivore);
		carnivores.getSelectionModel().select(0);

		ObservableList<String> optionsPlants = ImplementedLifeforms.getListImplementedPlants();
		ComboBox<String> plants = new ComboBox<>(optionsPlants);
		plants.getSelectionModel().select(0);
		int row=0;

		// Habitat implementation
		grid.add(new Label("Habitat Implementation"), 0, row);
		grid.add(habitats, 1, row++);
		// Plant implementation
		grid.add(new Label("Plant Implementation"), 0, row);
		grid.add(plants, 1, row++);
		// Herbivore implementation
		grid.add(new Label("Herbivore Implementation"), 0, row);
		grid.add(herbivores, 1, row++);
		// Carnivore implementation
		grid.add(new Label("Carnivore Implementation"), 0, row);
		grid.add(carnivores, 1, row++);

		// x dimension of domain
		ecosystemDimensionWidth = new IntegerTextField(2000);
		grid.add(new Label("Ecosystem Width"), 0, row);
		grid.add(ecosystemDimensionWidth, 1, row++);

		// y dimension of domain
		ecosystemDimensionHeight = new IntegerTextField(2000);
		grid.add(new Label("Ecosystem Height"), 0, row);
		grid.add(ecosystemDimensionHeight, 1, row++);

		// Continue to parameter scene
		Button sceneChanger = new Button("Continue");
		sceneChanger.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(ActionEvent event) {
				if (CheckIfTextFieldsAreEmpty()) {
					Alert alert = new Alert(AlertType.ERROR);
					alert.setTitle("Wrong parameters");
					alert.setHeaderText("Parameters were not set correctly");
					alert.setContentText("Ecosystem dimensions are required, aswell as lifeform implementations.\nHaving herbivores requires having plants, having carnivores requires having herbivores in the ecosystem");
					alert.showAndWait();
				} else {
					if (lastChosenPlant != null && lastChosenHerbivore != null && lastChosenCarnivore != null && lastChosenHabitat !=null
							&& lastChosenPlant.equals(plants.getSelectionModel().getSelectedItem())
							&& lastChosenHerbivore.equals(herbivores.getSelectionModel().getSelectedItem())
							&& lastChosenCarnivore.equals(carnivores.getSelectionModel().getSelectedItem())
							&& lastChosenHabitat.equals(habitats.getSelectionModel().getSelectedItem())) {
						window.setScene(setParametersScene);
					} else {

						lastChosenHabitat = habitats.getSelectionModel().getSelectedItem();
						lastChosenPlant = plants.getSelectionModel().getSelectedItem();
						lastChosenHerbivore = herbivores.getSelectionModel().getSelectedItem();
						lastChosenCarnivore = carnivores.getSelectionModel().getSelectedItem();

						ScrollPane scroll = new ScrollPane(); 
						
						Button goBack = new Button("Go Back");
						goBack.setOnAction(new EventHandler<ActionEvent>() {
							@Override
							public void handle(ActionEvent event) {
								window.setScene(firstScene);
							}
						});
						
						Button startSim = new Button("Continue");
						startSim.setOnAction(new EventHandler<ActionEvent>() {
							@Override
							public void handle(ActionEvent event) {
								simScene = createSimulationPane();
								window.setScene(simScene);
								window.sizeToScene();
							}
						});
						
						
						setParametersScene = new ParameterScene(scroll, 500, 450, lastChosenPlant, lastChosenHerbivore, lastChosenCarnivore,
								lastChosenHabitat, goBack, startSim);
						
						try {
							String current = new java.io.File( "." ).getCanonicalPath();
							setParametersScene.getStylesheets().add("artificialLifeSimulator/CSS/style.css");
						} catch (IOException e) {


						}
						window.setScene(setParametersScene);
						window.sizeToScene();
					}
				}
			}

			/**
			 * Method checks if all the required fields are filled
			 * @return true if the text fields are empty
			 */
			private boolean CheckIfTextFieldsAreEmpty() {
				if (ecosystemDimensionWidth.getText().equals("")) {
					return true;
				}
				if (ecosystemDimensionHeight.getText().equals("")) {
					return true;
				}
				if (plants.getSelectionModel().getSelectedItem().equals("None")
						|| !carnivores.getSelectionModel().getSelectedItem().equals("None") && herbivores.getSelectionModel().getSelectedItem().equals("None")) {
					return true;
				}
				return false;

			}
		});

		FileChooser fileChooser = new FileChooser();
		fileChooser.setTitle("Load");
		
		Button load = new Button("Load");
		load.setOnAction(
	            new EventHandler<ActionEvent>() {
	                @Override
	                public void handle(final ActionEvent e) {
	                	File file = fileChooser.showOpenDialog(window);
	                	try{
	                		FileInputStream fileIn = new FileInputStream(file.getAbsolutePath());
	                        ObjectInputStream in = new ObjectInputStream(fileIn);
	                        Biome biome = (Biome)in.readObject();
	                        in.close();
	                        fileIn.close();
                    		GridPane pane = new GridPane();
                    		if(widthWindow>biome.getEcosystemWidth()){
                    			widthWindow=biome.getEcosystemWidth();
                    		}
                    		if(heightWindow>biome.getEcosystemHeight()){
                    			heightWindow = biome.getEcosystemHeight();
                    		}
                        	simScene = new SimulationScene((Biome) biome, firstScene,
                    				window,widthWindow + GuiConstants.windowSizeModifierConstant, heightWindow, pane);

	            			String current = new java.io.File( "." ).getCanonicalPath();
	            			simScene.getStylesheets().add("artificialLifeSimulator/CSS/style.css");
	                        window.setScene(simScene);
	                        window.sizeToScene();
	                	}catch(Exception exception){
	                		System.out.println(exception);
	                		Alert alert = new Alert(AlertType.ERROR);
	    					alert.setTitle("Wrong file");
	    					alert.setHeaderText("File can't be loaded");
	    					alert.setContentText("Error can be caused by one of the following reasons:"
	    							+ "\nChosen file can't be read\n"
	    							+ "\nChosen file is not a serialized biome file"
	    							+ "\nChosen file is an outdated biome file");
	    					alert.showAndWait();
	                	}

	                }
	            });
		grid.add(load, 0, row);
		grid.add(sceneChanger, 1, row++);
	}

	/**
	 * Method used to properly finish the simulation
	 */
	private void closeProgram() {
		window.close();
		Runtime.getRuntime().halt(0);
	}

	/**
	 * Method is used to start the application
	 */
	public void startFrame() {
		launch();
	}

}
