package artificialLifeSimulator.GUI;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import artificialLifeSimulator.lifeForms.populations.Biome;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.layout.GridPane;
import javafx.stage.FileChooser;
import javafx.stage.Stage;


/**
 * @author Toni
 * Class implements methods used to interact with the drawing thread and simulating thread
 */
public class SimulationControlPane extends GridPane{
	private SimulationScene simScene;

	/**
	 * Constructor method is used to create the simulation control pane
	 * @param scene simulation scene reference
	 * @param firstScene references the first scene of the application where the basic parameters are being set up, and where the application returs when it exits the simulation
	 * @param stage references the stage of the application
	 */
	public SimulationControlPane(SimulationScene scene, Scene firstScene, Stage stage){
		super();
		simScene = scene;
		this.setHgap(10);
		this.setVgap(10);
		this.setPadding(new Insets(10, 10, 10, 10));

		Button save = new Button("Save");
		save.setOnAction(new EventHandler<ActionEvent>() {
	        public void handle(ActionEvent e) {
	            FileChooser fileChooser = new FileChooser();
	            fileChooser.setTitle("Save Biome");
	            File file = fileChooser.showSaveDialog(stage);
	            try
	            {
	               FileOutputStream fileOut =
	               new FileOutputStream(file);
	               ObjectOutputStream out = new ObjectOutputStream(fileOut);
	               Biome biome = scene.getCurrentBiome().copyOf();
	               out.writeObject(biome);
	               out.flush();
	               out.close();
	               fileOut.close();
	            }catch(IOException i)
	            {

	            }
	        }
	    });
		
		Button start = new Button("Start");
		start.setOnAction(new EventHandler<ActionEvent>() {
			private boolean running = false;
			private boolean firstTime = true;
			
			@Override
			public void handle(ActionEvent event) {
				if(!running && firstTime){
					running=true;
					firstTime = false;
					scene.startSimulation();
					start.setText("Stop");
				}else if(!running && !firstTime){
					running=true;
					scene.unblockThreads();
					start.setText("Stop");
				}else{
					start.setText("Start");
					scene.blockThreads();
					running=false;
				}
			}
			
		});
		
		Button exit = new Button("Exit");
		exit.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(ActionEvent event) {
				scene.terminateSimulation();
				scene.unblockThreads();
				stage.setScene(firstScene);
				stage.sizeToScene();
			}
			
		});
		int row = 0;
		this.add(start, 0, row++);
		this.add(save, 0, row++);
		this.add(exit, 0, row++);
		
	}
}
