package artificialLifeSimulator.GUI.controls;

import javafx.scene.control.IndexRange;
import javafx.scene.control.TextField;

/**
 * @author Toni
 * Class implements text check for input of a real number between 0 and 1 inclusive
 */
public class PercentageTextField extends TextField{
	
	/**
	 * Default constructor
	 */
	public PercentageTextField(){
		
	}
	/**
	 * Constructor sets the initial text to given text
	 * @param text initial text
	 */
	public PercentageTextField(String text){
		super(text);
	}
	
	@Override
    public void replaceText(int start, int end, String text)
    {
        if (validate(text,start,end))
        {
        	if(Double.parseDouble(this.getText()+text)>100){
        		super.replaceText(0, this.getLength(), "100");
        	}else{
        		super.replaceText(start, end, text);
        	}
        }
    }

    @Override
    public void replaceSelection(String text)
    {
        if (validateSelection(text))
        {
            super.replaceSelection(text);
        }
    }

    /**
     * Method is used to validate the given text
     * @param text text
     * @param start start where the text is being set
     * @param end end of the input text
     * @return true if the new text value is a valid 0-1 float
     */
    private boolean validate(String text,int start,int end)
    {
    	try{
    		if(!this.getText().equals(""))
    			Double.parseDouble(this.getText()+text);
    	}catch(NumberFormatException e){
    		return false;
    	}
    	IndexRange range = this.getSelection();
        return (this.getText(0,range.getStart())+text + this.getText(range.getEnd(),this.getLength())).matches("((0.[0-9]*)|(0|1))");
    }
    
    /**
     * Method is used to validate selection
     * @param text text to be validated
     * @return true if the new text value is a valid 0-1 float
     */
    private boolean validateSelection(String text)
    {
    	try{
    		if(!this.getText().equals(""))
    			Double.parseDouble(this.getText()+text);
    	}catch(NumberFormatException e){
    		return false;
    	}
    	IndexRange range = this.getSelection();
        return (this.getText(0,range.getStart())+this.getText(range.getEnd(),this.getLength())).matches("((0.[0-9]*)|(0|1))");
    }
}
