package artificialLifeSimulator.GUI.parameterScenes.basic;

import artificialLifeSimulator.CSS.CSS;
import artificialLifeSimulator.GUI.controls.IntegerTextField;
import artificialLifeSimulator.GUI.controls.PercentageTextField;
import artificialLifeSimulator.GUI.parameterScenes.AbstractHerbivoreParameterScene;
import artificialLifeSimulator.abstractClasses.AbstractCarnivorePopulation;
import artificialLifeSimulator.abstractClasses.AbstractHerbivorePopulation;
import artificialLifeSimulator.abstractClasses.AbstractPlant;
import artificialLifeSimulator.abstractClasses.AbstractPlantPopulation;
import artificialLifeSimulator.lifeForms.populations.BasicHerbivorePopulation;
import artificialLifeSimulator.lifeForms.populations.BasicPlantPopulation;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.layout.GridPane;

/**
 * @author Toni
 * Class is used to creaate basic herbivore parameter scene
 */
public class BasicHerbivoreParameterScene extends AbstractHerbivoreParameterScene {
	
	private Label maxHerbivores;
	private IntegerTextField maxHerbivoresValue;
	
	private Label initialHerbivorePopulationSize;
	private IntegerTextField initialHerbivorePopulationSizeValue;

	private Label maxAge;
	private IntegerTextField maxAgeValue;
	
	private Label maxEnergy;
	private IntegerTextField maxEnergyValue;
	
	private Label maxSpeed;
	private IntegerTextField maxSpeedValue;
	
	private Label energyForReproduction;
	private IntegerTextField energyForReproductionValue;

	private Label startingEnergy;
	private IntegerTextField startingEnergyValue;
	
	private Label chanceOfReproduction;
	private PercentageTextField chanceOfReproductionValue;
	
	private Label ageRatePerIteration;
	private IntegerTextField ageRatePerIterationValue;

	/**
	 * Constructor for basic herbivore parameter scene
	 */
	public BasicHerbivoreParameterScene(){
		initializeScene();
	}
	
	@Override
	public AbstractHerbivorePopulation createPopulation(int ecosystemWidth,int ecosystemHeight, AbstractPlantPopulation<AbstractPlant> plants) {
		if(!checkFields()){
			return null;
		}
		return new BasicHerbivorePopulation(plants,Integer.parseInt(maxHerbivoresValue.getText())
				, Integer.parseInt(initialHerbivorePopulationSizeValue.getText()), 
				Double.parseDouble(maxSpeedValue.getText()),
				Double.parseDouble(maxEnergyValue.getText()), 
				Double.parseDouble(startingEnergyValue.getText()),
				Double.parseDouble(chanceOfReproductionValue.getText()),
				Double.parseDouble(ageRatePerIterationValue.getText()),
				Double.parseDouble(maxAgeValue.getText()),
				Double.parseDouble(energyForReproductionValue.getText()),
				ecosystemWidth,
				ecosystemHeight);
	}

	private boolean checkFields() {
		try{
			int popSize1,popSize2;
			double doubleTester1,doubleTester2;
			popSize1=Integer.parseInt(maxHerbivoresValue.getText());
			popSize2=Integer.parseInt(initialHerbivorePopulationSizeValue.getText()); 
			if(popSize2>popSize1)
				return false;
			doubleTester1=Double.parseDouble(maxEnergyValue.getText());
			doubleTester2=Double.parseDouble(startingEnergyValue.getText());
			if(doubleTester1<doubleTester2 || popSize1<popSize2){
				return false;
			}
			doubleTester1=Double.parseDouble(chanceOfReproductionValue.getText());
			doubleTester1=Integer.parseInt(ageRatePerIterationValue.getText());
		}catch(Exception e){
			
			return false;
		}
			return true;
	}

	@Override
	public void initializeScene() {
		
		maxHerbivores = new Label("Maximum herbivore population size");
		maxHerbivoresValue = new IntegerTextField("50");
		
		initialHerbivorePopulationSize = new Label("Initial herbivore popoulation size");
		initialHerbivorePopulationSizeValue = new IntegerTextField("5");
		
		maxEnergy = new Label("Maximum herbivore energy");
		maxEnergyValue = new IntegerTextField("200");

		startingEnergy = new Label("Starting herbivore energy");
		startingEnergyValue = new IntegerTextField("20");
		
		maxSpeed = new Label("Maximum herbivore speed");
		maxSpeedValue = new IntegerTextField("5");
		
		energyForReproduction = new Label("Energy required for reproduction");
		energyForReproductionValue = new IntegerTextField("100");
		
		maxAge = new Label("Maximum age");
		maxAgeValue = new IntegerTextField("1000");

		chanceOfReproduction = new Label("Chance of reproduction");
		chanceOfReproductionValue = new PercentageTextField("0.05");

		ageRatePerIteration = new Label("Aging rate");
		ageRatePerIterationValue = new IntegerTextField("1");
	}

	@Override
	public int createForm(GridPane pane, int currentRow) {
		pane.setAlignment(Pos.TOP_LEFT);
		pane.setHgap(10);
		pane.setVgap(10);
		pane.add(maxHerbivores, 0, currentRow);
		pane.add(maxHerbivoresValue, 1, currentRow++);
		pane.add(initialHerbivorePopulationSize, 0, currentRow);
		pane.add(initialHerbivorePopulationSizeValue, 1, currentRow++);
		pane.add(maxSpeed, 0, currentRow);
		pane.add(maxSpeedValue, 1, currentRow++);
		pane.add(maxEnergy, 0, currentRow);
		pane.add(maxEnergyValue, 1, currentRow++);
		pane.add(startingEnergy, 0, currentRow);
		pane.add(startingEnergyValue, 1, currentRow++);
		pane.add(chanceOfReproduction, 0, currentRow);
		pane.add(chanceOfReproductionValue, 1, currentRow++);
		pane.add(energyForReproduction, 0, currentRow);
		pane.add(energyForReproductionValue, 1, currentRow++);
		pane.add(ageRatePerIteration, 0, currentRow);
		pane.add(ageRatePerIterationValue, 1, currentRow++);
		pane.add(maxAge, 0, currentRow);
		pane.add(maxAgeValue, 1, currentRow++);
		
		return currentRow;
		
	}

}
