package artificialLifeSimulator.abstractClasses;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.Random;

import artificialLifeSimulator.lifeForms.basic.BasicPlant;
import javafx.scene.canvas.GraphicsContext;
/**
 * @author Toni
 * Class is defines methods that are used to simulate the behavior of plant population in the ecosystem
 *
 * @param <T> Parameter T must be a class that extends the AbstractPlant class
 */
public abstract class AbstractPlantPopulation<T extends AbstractPlant> implements Serializable{
	/**
	 * List of classes that extend AbstractPlant class
	 */
	protected LinkedList<T> population;
	
	/**
	 * Method is used to simulate changes of the plant population during time in the ecosystem, which are represented as iterations
	 * @param ecosystemWidth ecosystem width
	 * @param ecosystemHeight ecosystem height
	 * @param rand random number generator
	 */
	public abstract void nextIteration(int ecosystemWidth, int ecosystemHeight, Random rand);
	/**
	 * Method is used to create a full copy of the plant population
	 * @return copy of the plant population
	 */
	public abstract AbstractPlantPopulation<T> copyOf();

	/** 
	 * Method is used to get the plant population
	 * @return the copy of the population
	 */
	public LinkedList<T> getPopulation(){
		return this.population;
	}
	
	/**
	 * Method is used to draw the abstract plant population
	 * @param g Graphics context used to draw the population
	 */
	public void drawPopulation(GraphicsContext g){
		for (T temp : population) {
			temp.drawMe(g);
		}
	}
}
